# Copyright 2024 Oreum Industries
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Core tools for use on projects by Oreum Industries"""

import logging

__version__ = "0.10.6"

# logger goes to null handler by default
# packages that import oreum_core can override this and direct elsewhere
# e.g.
#   logger = logging.getLogger('oreum_core')
#   logger.addHandler(logging.FileHandler('log.log', mode='a', encoding='utf-8'))
logging.getLogger("oreum_core").addHandler(logging.NullHandler())
